from kafka import KafkaConsumer
from json import loads

def create_consumer(topic_name: str, bootstrap_server_list: list) -> KafkaConsumer:    
        consumer = KafkaConsumer(
                topic_name,
                bootstrap_servers=bootstrap_server_list,
                value_deserializer=lambda x: loads(x.decode('utf-8')),
                #consumer_timeout_ms=5000,
                auto_offset_reset='earliest', #'earliest' 'latest'
                # group_id="fbi",
                # enable_auto_commit=True,
        )

        return consumer
        



from textual import on
from textual.app import App, ComposeResult
from textual.screen import Screen
from textual.containers import Container, Vertical, Center, Middle
from textual.widgets import (
  Input, RichLog, Button, Sparkline,
  Header, Footer
)
from textual.worker import Worker
from textual import work

from pytxtchat.kafka.producer import create_producer
from pytxtchat.kafka.consumer import create_consumer

from textual.reactive import reactive

from rich.text import Text
from rich.panel import Panel

import random

# 
CHAT_ROOM = ""
CHAT_ID = ""
CHAT_SERVERS=""

class HomeScreen(Screen):
    
    def compose(self) -> ComposeResult:
        yield Container(
            Input(placeholder="Chat Room", classes="chat_setting_input_widget", id="chat_room_input_widget"),
            Input(placeholder="Chat ID", classes="chat_setting_input_widget", id="chat_id_input_widget"),
            Input(placeholder="Chat Server A:9092,B:9092", classes="chat_setting_input_widget", id="chat_servers_input_widget"),
            Button("Jumping into the world", id="get_started", classes="button_widget"),
            Button("Backward step", id="exit", classes="button_widget"),
            classes="home_screen_container",
        )
        
    @on(Button.Pressed)
    def button_pressed_handler(self, event: Button.Pressed) -> None:
        if event.button.id == "get_started":
            global CHAT_ROOM, CHAT_ID, CHAT_SERVERS
            CHAT_ROOM = self.query_one("#chat_room_input_widget", Input).value
            CHAT_ID = self.query_one("#chat_id_input_widget", Input).value
            CHAT_SERVERS = self.query_one("#chat_servers_input_widget", Input).value.split(",")
            self.app.push_screen(ChatScreen())
        elif event.button.id == "exit":
            self.app.exit()
            
            
class ChatScreen(Screen):
    def on_mount(self) -> None:
        
        self.load_msg()
        self.notify(
            message="You have entered successfully."
            "[b]conversation[/b] and [i][b]communication[/b][/i] are alive!",
            title="Welcome Chat World 🥳",
            severity="information", #warning #error
        )
        self.kafka_producer = create_producer(CHAT_SERVERS)
        self.screen.styles.border = ("heavy", "white")

    def compose(self) -> ComposeResult:
        yield Header(
            show_clock = True,
            icon = "💬"
        )
        yield Footer(
            show_command_palette = False
        )
        yield Vertical(
            RichLog(classes="chat_history_widget", min_width=self.app.size.width - 6),
            Input(placeholder="Enter chat", classes="chat_input_widget"),
            classes="vertical_layout"
        )

    @on(Input.Submitted)
    def input_submitted_handler(self, event: Input.Submitted):
        
        msg = {
            "to" : CHAT_ID,
            "msg" : event.value
        }

        self.kafka_producer.send(topic=CHAT_ROOM, value=msg)
        self.kafka_producer.flush()

        input = self.query_one(Input)
        input.value = "" 

    @work(exclusive=True, thread=True)
    def load_msg(self) -> None:
        """https://textual.textualize.io/guide/workers"""
        consumer = create_consumer(topic_name=CHAT_ROOM, bootstrap_server_list=CHAT_SERVERS)

        chat_history_widget = self.query_one(RichLog)
        for m in consumer:
            data = m.value
            to = data['to']
            msg = data['msg']

            if to == CHAT_ID:
                justify_value = "right"
                style_value = "bold white"
            else:
                justify_value = "left"
                style_value = "bold green"
 
            text_prod = Text(f"{to}: {msg}", style=style_value, justify=justify_value)
            chat_history_widget.write(text_prod)


class ChatApp(App):
    CSS_PATH = "chat.tcss"
    
    def on_mount(self) -> None:
        self.app.push_screen(HomeScreen())
        self.title = "Terminal-based chat with Kafka and Python"

if __name__  == "__main__":
    app=ChatApp()
    app.run()


from pytxtchat.com import pong
from pytxtchat.textual.stopwatch import StopwatchApp
from pytxtchat.textual.chat import ChatApp
from pytxtchat.textual.weather import WeatherApp

from pytxtchat.data.manager import (
    get_parquet_path, 
    search_movie,
    show_group_count)

import typer, fire
import requests

def ping():
    msg = pong()
    print(msg)
    
def run_chat():
    app = ChatApp()
    app.run()

def run_stopwatch():
    app = StopwatchApp()
    app.run()

def run_weather():
    app = WeatherApp()
    app.run()

def weather_fire():
    fire.Fire(print_weather)

def weather_typer():
     typer.run(print_weather)
     

def print_weather(city: str, lang='ko') -> None:
    """날씨를 알려드립니다.
    https://wttr.in/:help
    
    Args:
        city: 도시이름
        lang: 표현언어
    """
    url = f"https://wttr.in/{city}?lang={lang}"
    response = requests.get(url)
    print(response.text)

def print_parquet_path():
    typer.run(get_parquet_path)

def print_search_movie():
     fire.Fire(search_movie)

def print_group_count():
    typer.run(show_group_count)


import os
import pandas as pd
import termplotlib as tpl
from tabulate import tabulate

def get_parquet_path(dname='movies', isprint='True') -> str:
    """parquet 파일 경로를 반환

    Args:
        dname (str, optional): 데이터 종류 (movies, moviedetail 등). 기본값은 'movies'.
        isprint (str, optional): 경로를 출력할지 여부. 기본값은 True.

    Returns:
        str: parquet 파일의 절대 경로

    이 함수는 주어진 데이터 종류에 해당하는 파라쿼 파일의 절대 경로를 생성하고,
    isprint 가 True일 경우 경로를 출력합니다.
    """
    parquet_path = os.path.join(os.path.dirname(__file__), dname, f"{dname}.parquet")
    if isprint.lower() == 'true':
       print(parquet_path)
    return parquet_path


def get_dataframe(dname='movies') -> pd.DataFrame:
    """parquet 파일 경로를 반환

    Args:
        dname (str, optional): 데이터 종류 (movies, moviedetail 등). 기본값은 'movies'

    Returns:
        DataFrame: pandas dataframe

    이 함수는 주어진 데이터 종류에 해당하는 pandas dataframe 을 반환합니다.
    """
    p = get_parquet_path(dname=dname, isprint='False')
    df = pd.read_parquet(p)
    return df


def search_movie(word: str, column='movieNm', isprint='True', tablefmt='github'):
    """영화 검색 함수

    Args:
      word: 검색할 단어
      column: 검색할 컬럼 (기본값: 'movieNm') nationAlt, genreAlt, typeNm, directorNm, companyNm
      isprint: 검색 결과를 출력할지 여부 (기본값: 'True')
      tablefmt: 출력 형식 (기본값: 'github') plain, simple, grid, pipe, pretty, tsv ... https://pypi.org/project/tabulate/

    Returns:
      검색 결과를 담은 문자열 (테이블 형식)
    """
    df = get_dataframe()
    s = df[df[column].str.contains(word, na=False)]
    cols = ['movieNm', 'openDt', 'nationAlt', 'genreAlt', 'typeNm', 'directorNm', 'companyNm', 'movieCd', 'year']
    r = s[cols].sort_values(by=['movieNm', 'openDt'], ascending=[True, True])
    t = tabulate(r, headers=cols, tablefmt=tablefmt)
    if isprint.lower() == 'true':
       print(t)
    return t

def show_group_count(column='year', star="False", limit=10):
    """
    특정 컬럼을 기준으로 그룹화하여 데이터 개수를 세고 시각화하는 함수

    Args:
      column (str, optional): 그룹화할 컬럼 명 (기본값: 'year').
      star (str, optional): 그래프 축 라벨에 별 문자 사용 여부 (기본값: "False"). 'True'이면 사용, 'False'이면 사용하지 않음.
      limit (int, optional): 상위 N개 그룹만 표시할지 여부 (기본값: 10).

    Returns:
      None
    """
    df = get_dataframe()
    gdf = df.groupby(column).size().to_frame(name='cnt').sort_values('cnt', ascending=False)

    if len(gdf) > 10:
        gdf = gdf.head(int(limit))
        
    
    fdf = gdf.sort_values(column, ascending=True).reset_index()
    fig = tpl.figure()
    fig.barh(fdf['cnt'], fdf[column], force_ascii=(star.lower() == 'true'))
    fig.show()



from kafka import KafkaProducer
import json

def create_producer(bootstrap_server_list: list) -> KafkaProducer:
    pro = KafkaProducer(
        bootstrap_servers=bootstrap_server_list,
        value_serializer=lambda x:json.dumps(x).encode('utf-8')
        )
    return pro
    

# def send_msg(topic: str, msg: dict):
#     pro.send(topic=topic, value=msg)
#     pro.flush()
